﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace Ellab.Charting
{
    public class Annotation : ITool
    {
        private Steema.TeeChart.Tools.Annotation tAnnotation;

        #region Events

        public event EventHandler TextChanged;
        public event EventHandler SizeChanged;
        public event EventHandler PositionChanged;

        #endregion

        public Chart Chart
        {
            get;
            private set;
        }

        public string Name
        {
            get;
            private set;
        }

        public ToolType Type { get { return ToolType.Annotation; } }

        #region Text

        public string Text
        {
            get { return tAnnotation.Text; }
            set
            {
                if (textDelegate == null)
                    textDelegate = new Action<string>(SetText);
                Chart.InvokeDelegate(textDelegate, value);
            }
        }
        private Action<string> textDelegate = null;
        private void SetText(string value)
        {
            tAnnotation.Text = value;
            if (TextChanged != null)
                TextChanged(this, new EventArgs());
        }

        public class AnnotationFont
        {
            private Annotation parent;
            internal AnnotationFont(Annotation _parent)
            {
                parent = _parent;
            }

            public bool Bold
            {
                get
                {
                    return parent.tAnnotation.Shape.Font.Bold;
                }
                set
                {
                    if (boldDelegate == null)
                        boldDelegate = new Action<bool>(SetBold);
                    parent.Chart.InvokeDelegate(boldDelegate, value);
                }
            }
            private Action<bool> boldDelegate = null;
            private void SetBold(bool value)
            {
                parent.tAnnotation.Shape.Font.Bold = value;
            }

            public bool Italic
            {
                get { return parent.tAnnotation.Shape.Font.Italic; }
                set
                {
                    if (italicDelegate == null)
                        italicDelegate = new Action<bool>(SetItalic);
                    parent.Chart.InvokeDelegate(italicDelegate, value);
                }
            }
            private Action<bool> italicDelegate = null;
            private void SetItalic(bool value)
            {
                parent.tAnnotation.Shape.Font.Italic = value;
            }

            public int Size
            {
                get { return parent.tAnnotation.Shape.Font.Size; }
                set
                {
                    if (sizeDelegate == null)
                        sizeDelegate = new Action<int>(SetSize);
                    parent.Chart.InvokeDelegate(sizeDelegate, value);
                }
            }
            private Action<int> sizeDelegate = null;
            private void SetSize(int value)
            {
                parent.tAnnotation.Shape.Font.Size = value;
            }

            public Color Color
            {
                get { return parent.tAnnotation.Shape.Font.Color; }
                set
                {
                    if (colorDelegate == null)
                        colorDelegate = new Action<Color>(SetColor);
                    parent.Chart.InvokeDelegate(colorDelegate, value);
                }
            }
            private Action<Color> colorDelegate = null;
            private void SetColor(Color value)
            {
                parent.tAnnotation.Shape.Font.Color = value;
            }
        }
        private AnnotationFont font = null;
        public AnnotationFont Font
        {
            get
            {
                if (font == null)
                    font = new AnnotationFont(this);
                return font;
            }
        }

        #endregion Text

        #region Size

        public bool AutoSize
        {
            get { return tAnnotation.Shape.AutoSize; }
            set
            {
                if (autoSizeDelegate == null)
                    autoSizeDelegate = new Action<bool>(SetAutoSize);
                Chart.InvokeDelegate(autoSizeDelegate, value);
            }
        }
        private Action<bool> autoSizeDelegate = null;
        private void SetAutoSize(bool value)
        {
            tAnnotation.Shape.AutoSize = value;
        }

        public Size Size
        {
            get
            {
                return new Size(tAnnotation.Shape.Right - tAnnotation.Shape.Left, tAnnotation.Shape.Bottom - tAnnotation.Shape.Top);
            }
            set
            {
                if (sizeDelegate == null)
                    sizeDelegate = new Action<Size>(SetSize);
                Chart.InvokeDelegate(sizeDelegate, value);
            }
        }
        private Action<Size> sizeDelegate = null;
        private void SetSize(Size value)
        {
            tAnnotation.Shape.Right = tAnnotation.Shape.Left + value.Width;
            tAnnotation.Shape.Bottom = tAnnotation.Shape.Top + value.Height;

            if (SizeChanged != null)
                SizeChanged(this, new EventArgs());
        }

        #endregion

        public Rectangle Bounds
        {
            get { return new Rectangle(PixelPosition, Size); }
        }

        #region Position

        public Point PixelPosition
        {
            get
            {
                return new Point(tAnnotation.Shape.Left, tAnnotation.Shape.Top);
            }
            set
            {
                if (pixelPositionDelegate == null)
                    pixelPositionDelegate = new Action<Point>(SetPixelPosition);
                Chart.InvokeDelegate(pixelPositionDelegate, value);
            }
        }
        private Action<Point> pixelPositionDelegate = null;
        private void SetPixelPosition(Point value)
        {
            tAnnotation.Shape.Right = value.X + Size.Width;
            tAnnotation.Shape.Left = value.X;
            tAnnotation.Shape.Bottom = value.Y + Size.Height;
            tAnnotation.Shape.Top = value.Y;

            if (PositionChanged != null)
                PositionChanged(this, new EventArgs());
        }

        public int PixelPositionX
        {
            get { return tAnnotation.Shape.Left; }
            set
            {
                if (pixelPositionXDelegate == null)
                    pixelPositionXDelegate = new Action<int>(SetPixelPositionX);
                Chart.InvokeDelegate(pixelPositionXDelegate, value);
            }
        }
        private Action<int> pixelPositionXDelegate = null;
        private void SetPixelPositionX(int value)
        {
            tAnnotation.Shape.Right = value + Size.Width;
            tAnnotation.Shape.Left = value;

            if (PositionChanged != null)
                PositionChanged(this, new EventArgs());
        }

        public int PixelPositionY
        {
            get { return tAnnotation.Shape.Top; }
            set
            {
                if (pixelPositionYDelegate == null)
                    pixelPositionYDelegate = new Action<int>(SetPixelPositionY);
                Chart.InvokeDelegate(pixelPositionYDelegate, value);
            }
        }
        private Action<int> pixelPositionYDelegate = null;
        private void SetPixelPositionY(int value)
        {
            tAnnotation.Shape.Bottom = value + Size.Height;
            tAnnotation.Shape.Top = value;

            if (PositionChanged != null)
                PositionChanged(this, new EventArgs());
        }

        ///// <summary>
        ///// Doesn't work as intended. TeeChart returns incorrect values.
        ///// </summary>
        ///// <param name="series"></param>
        ///// <param name="xValue"></param>
        ///// <param name="yValue"></param>
        //public void SetValuePosition(ISeries series, DateTime xValue, double yValue)
        //{
        //    //int x = Chart.HorizontalAxis.ValueToPixelCoordinate(xValue.ToOADate());
        //    //int y = yAxis.ValueToPixelCoordinate(yValue);
        //    int x = series.HorizontalAxis.TAxis.CalcXPosValue(xValue.ToOADate());
        //    int y = series.VerticalAxis.TAxis.CalcYPosValue(yValue);

        //    PixelPosition = new Point(x, y);
        //}

        #endregion

        public bool Visible
        {
            get { return tAnnotation.Active; }
            set
            {
                if (visibleDelegate == null)
                    visibleDelegate = new Action<bool>(SetVisible);
                Chart.InvokeDelegate(visibleDelegate, value);
            }
        }
        private Action<bool> visibleDelegate = null;
        private void SetVisible(bool value)
        {
            tAnnotation.Active = value;
        }

        #region Shadow

        public class AnnotationShadow
        {
            private Annotation parent;
            internal AnnotationShadow(Annotation _parent)
            {
                parent = _parent;
            }

            public bool Visible
            {
                get { return parent.tAnnotation.Shape.Shadow.Visible; }
                set
                {
                    if (visibleDelegate == null)
                        visibleDelegate = new Action<bool>(SetVisible);
                    parent.Chart.InvokeDelegate(visibleDelegate, value);
                }
            }
            private Action<bool> visibleDelegate;
            private void SetVisible(bool value)
            {
                parent.tAnnotation.Shape.Shadow.Visible = value;
            }
        }
        private AnnotationShadow shadow = null;
        public AnnotationShadow Shadow
        {
            get
            {
                if (shadow == null)
                    shadow = new AnnotationShadow(this);
                return shadow;
            }
        }

        #endregion Shadow

        #region Background

        public class AnnotationBackground
        {
            private Annotation parent;
            internal AnnotationBackground(Annotation _parent)
            {
                parent = _parent;
            }

            public Color Color
            {
                get { return parent.tAnnotation.Shape.Color; }
                set
                {
                    if (colorDelegate == null)
                        colorDelegate = new Action<Color>(SetColor);
                    parent.Chart.InvokeDelegate(colorDelegate, value);
                }
            }
            private Action<Color> colorDelegate = null;
            private void SetColor(Color value)
            {
                parent.tAnnotation.Shape.Color = value;
            }

            public bool Visible
            {
                get { return parent.tAnnotation.Shape.Visible; }
                set
                {
                    if (visibleDelegate == null)
                        visibleDelegate = new Action<bool>(SetVisible);

                }
            }
            private Action<bool> visibleDelegate = null;
            private void SetVisible(bool value)
            {
                parent.tAnnotation.Shape.Visible = value;
            }

            public int Transparency
            {
                get { return parent.tAnnotation.Shape.Transparency; }
                set
                {
                    if (transparencyDelegate == null)
                        transparencyDelegate = new Action<int>(SetTransparency);
                    parent.Chart.InvokeDelegate(transparencyDelegate, value);
                }
            }
            private Action<int> transparencyDelegate = null;
            private void SetTransparency(int value)
            {
                parent.tAnnotation.Shape.Transparency = value;
            }

            #region Gradient

            public class AnnotationBackgroundGradient
            {
                private Annotation parent;
                internal AnnotationBackgroundGradient(Annotation _parent)
                {
                    parent = _parent;
                }

                public bool Enabled
                {
                    get { return parent.tAnnotation.Shape.Gradient.Visible; }
                    set
                    {
                        if (enabledDelegate == null)
                            enabledDelegate = new Action<bool>(SetEnabled);
                        parent.Chart.InvokeDelegate(enabledDelegate, value);
                    }
                }
                private Action<bool> enabledDelegate = null;
                private void SetEnabled(bool value)
                {
                    parent.tAnnotation.Shape.Gradient.Visible = value;
                }

                public bool MiddleColorEnabled
                {
                    get { return parent.tAnnotation.Shape.Gradient.UseMiddle; }
                    set
                    {
                        if (middleColorEnabledDelegate == null)
                            middleColorEnabledDelegate = new Action<bool>(SetMiddleColorEnabled);
                        parent.Chart.InvokeDelegate(middleColorEnabledDelegate, value);
                    }
                }
                private Action<bool> middleColorEnabledDelegate = null;
                private void SetMiddleColorEnabled(bool value)
                {
                    parent.tAnnotation.Shape.Gradient.UseMiddle = value;
                }

                public Color StartColor
                {
                    get { return parent.tAnnotation.Shape.Gradient.StartColor; }
                    set
                    {
                        if (startColorDelegate == null)
                            startColorDelegate = new Action<Color>(SetStartColor);
                        parent.Chart.InvokeDelegate(startColorDelegate, value);
                    }
                }
                private Action<Color> startColorDelegate = null;
                private void SetStartColor(Color value)
                {
                    parent.tAnnotation.Shape.Gradient.StartColor = value;
                }

                public Color MiddleColor
                {
                    get { return parent.tAnnotation.Shape.Gradient.MiddleColor; }
                    set
                    {
                        if (middleColorDelegate == null)
                            middleColorDelegate = new Action<Color>(SetMiddleColor);
                        parent.Chart.InvokeDelegate(middleColorDelegate, value);
                    }
                }
                private Action<Color> middleColorDelegate = null;
                private void SetMiddleColor(Color value)
                {
                    parent.tAnnotation.Shape.Gradient.MiddleColor = value;
                }

                public Color EndColor
                {
                    get { return parent.tAnnotation.Shape.Gradient.EndColor; }
                    set
                    {
                        if (endColorDelegate == null)
                            endColorDelegate = new Action<Color>(SetEndColor);
                        parent.Chart.InvokeDelegate(endColorDelegate, value);
                    }
                }
                private Action<Color> endColorDelegate = null;
                private void SetEndColor(Color value)
                {
                    parent.tAnnotation.Shape.Gradient.EndColor = value;
                }
            }
            private AnnotationBackgroundGradient gradient = null;
            public AnnotationBackgroundGradient Gradient
            {
                get
                {
                    if (gradient == null)
                        gradient = new AnnotationBackgroundGradient(parent);
                    return gradient;
                }
            }

            #endregion Gradient

        }
        private AnnotationBackground background = null;
        public AnnotationBackground Background
        {
            get
            {
                if (background == null)
                    background = new AnnotationBackground(this);
                return background;
            }
        }

        #endregion Background

        #region Arrow

        public class AnnotationArrow
        {
            private Annotation parent;
            internal AnnotationArrow(Annotation _parent)
            {
                parent = _parent;
            }

            public bool Visible
            {
                get
                {
                    return parent.tAnnotation.Callout.Visible;
                }
                set
                {
                    if (visibleDelegate == null)
                        visibleDelegate = new Action<bool>(SetVisible);
                    parent.Chart.InvokeDelegate(visibleDelegate, value);
                }
            }
            private Action<bool> visibleDelegate = null;
            private void SetVisible(bool value)
            {
                parent.tAnnotation.Callout.Visible = value;
                parent.tAnnotation.Callout.Arrow.Visible = value;
            }

            public Point PixelPosition
            {
                get
                {
                    return new Point(parent.tAnnotation.Callout.XPosition, parent.tAnnotation.Callout.YPosition);
                }
                set
                {
                    if (pixelPositionDelegate == null)
                        pixelPositionDelegate = new Action<Point>(SetPixelPosition);
                    parent.Chart.InvokeDelegate(pixelPositionDelegate, value);
                }
            }
            private Action<Point> pixelPositionDelegate = null;
            private void SetPixelPosition(Point value)
            {
                parent.tAnnotation.Callout.XPosition = value.X;
                parent.tAnnotation.Callout.YPosition = value.Y;
                parent.tAnnotation.Callout.ZPosition = 0;
            }

            public int PixelPositionX
            {
                get { return parent.tAnnotation.Callout.XPosition; }
                set
                {
                    if (pixelPositionXDelegate == null)
                        pixelPositionXDelegate = new Action<int>(SetPixelPositionX);
                    parent.Chart.InvokeDelegate(pixelPositionXDelegate, value);
                }
            }
            private Action<int> pixelPositionXDelegate = null;
            private void SetPixelPositionX(int value)
            {
                parent.tAnnotation.Callout.XPosition = value;
                parent.tAnnotation.Callout.ZPosition = 0;
            }

            public int PixelPositionY
            {
                get { return parent.tAnnotation.Callout.YPosition; }
                set
                {
                    if (pixelPositionYDelegate == null)
                        pixelPositionYDelegate = new Action<int>(SetPixelPositionY);
                    parent.Chart.InvokeDelegate(pixelPositionYDelegate, value);
                }
            }
            private Action<int> pixelPositionYDelegate = null;
            private void SetPixelPositionY(int value)
            {
                parent.tAnnotation.Callout.YPosition = value;
                parent.tAnnotation.Callout.ZPosition = 0;
            }

            public Color Color
            {
                get { return parent.tAnnotation.Callout.Color; }
                set
                {
                    if (colorDelegate == null)
                        colorDelegate = new Action<Color>(SetColor);
                    parent.Chart.InvokeDelegate(colorDelegate, value);
                }
            }
            private Action<Color> colorDelegate = null;
            private void SetColor(Color value)
            {
                parent.tAnnotation.Callout.Color = value;
            }

            public class AnnotationArrowOutline
            {
                Annotation parent;
                internal AnnotationArrowOutline(Annotation _parent)
                {
                    parent = _parent;
                }

                public bool Visible
                {
                    get { return parent.tAnnotation.Callout.Arrow.Transparency > 0; }
                    set
                    {
                        if (visibleDelegate == null)
                            visibleDelegate = new Action<bool>(SetVisible);
                        parent.Chart.InvokeDelegate(visibleDelegate, value);
                    }
                }
                private Action<bool> visibleDelegate = null;
                private void SetVisible(bool value)
                {
                    if (value)
                        parent.tAnnotation.Callout.Arrow.Transparency = 0;
                    else
                        parent.tAnnotation.Callout.Arrow.Transparency = 100;
                }

                public int Width
                {
                    get { return parent.tAnnotation.Callout.Arrow.Width; }
                    set
                    {
                        if (widthDelegate == null)
                            widthDelegate = new Action<int>(SetWidth);
                        parent.Chart.InvokeDelegate(widthDelegate, value);
                    }
                }
                private Action<int> widthDelegate = null;
                private void SetWidth(int value)
                {
                    parent.tAnnotation.Callout.Arrow.Width = value;
                }

                public Color Color
                {
                    get { return parent.tAnnotation.Callout.Arrow.Color; }
                    set
                    {
                        if (colorDelegate == null)
                            colorDelegate = new Action<System.Drawing.Color>(SetColor);
                        parent.Chart.InvokeDelegate(colorDelegate, value);
                    }
                }
                private Action<Color> colorDelegate = null;
                private void SetColor(Color value)
                {
                    parent.tAnnotation.Callout.Arrow.Color = value;
                }
            }
            private AnnotationArrowOutline outline = null;
            public AnnotationArrowOutline Outline
            {
                get
                {
                    if (outline == null)
                        outline = new AnnotationArrowOutline(parent);
                    return outline;
                }
            }
        }
        private AnnotationArrow arrow = null;
        public AnnotationArrow Arrow
        {
            get
            {
                if (arrow == null)
                    arrow = new AnnotationArrow(this);
                return arrow;
            }
        }

        #endregion

        private Annotation(Chart _chart, string name, string text)
        {
            Chart = _chart;
            tAnnotation = new Steema.TeeChart.Tools.Annotation(Chart.TChart.Chart);
            tAnnotation.Shape.CustomPosition = true;

            tAnnotation.Callout.ArrowHead = Steema.TeeChart.Styles.ArrowHeadStyles.Solid;
            Arrow.Outline.Color = Color.Black;

            Background.Color = Color.White;
            Background.Gradient.StartColor = Color.White;
            Background.Gradient.EndColor = Color.White;

            Name = name;
            Text = text;

            Chart.AddTool(this);
        }

        public static Annotation Create(Chart _chart, string text)
        {
            int i = 1;
            while (_chart.Tools.Contains("Annotation " + i))
                i++;
            return Create(_chart, "Annotation " + i, text);
        }
        public static Annotation Create(Chart _chart, string name, string text)
        {
            return new Annotation(_chart, name, text);
        }

        public ITool CreateMirror(Chart _chart)
        {
            Annotation a = Annotation.Create(_chart, Name, Text);
            a.AutoSize = AutoSize;
            a.PixelPosition = PixelPosition;
            a.Size = Size;

            return a;
        }

        public void Remove()
        {
            if (removeDelegate == null)
                removeDelegate = new Action(RemoveInternal);
            Chart.InvokeDelegate(removeDelegate);
        }
        private Action removeDelegate = null;
        private void RemoveInternal()
        {
            Chart.TChart.Tools.Remove(tAnnotation);
            Chart.RemoveTool(this);
        }
    }
}