﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace Ellab.Charting
{
    public class Axis
    {
        private Chart chart;

        internal Steema.TeeChart.Axis TAxis
        {
            get;
            private set;
        }

        private string _unit = null;
        public string Unit
        {
            get { return _unit; }
            private set 
            { 
                _unit = value;
            }
        }

        public string Title
        {
            get { return TAxis.Title.Text; }
            set
            {
                if (titleDelegate == null)
                    titleDelegate = new Action<string>(SetTitle);
                chart.InvokeDelegate(titleDelegate, value);
            }
        }
        private Action<string> titleDelegate = null;
        private void SetTitle(string value)
        {
            TAxis.Title.Text = value;
        }

        public bool IsHorizontal
        {
            get { return TAxis.Horizontal; }
        }
        public bool IsVertical
        {
            get { return !TAxis.Horizontal; }
        }

        public string LabelFormat
        {
            get
            {
                if (TAxis.Horizontal)
                    return TAxis.Labels.DateTimeFormat;
                else
                    return TAxis.Labels.ValueFormat;
            }
            set
            {
                if (labelFormatDelegate == null)
                    labelFormatDelegate = new Action<string>(SetLabelFormat);
                chart.InvokeDelegate(labelFormatDelegate, value);
            }
        }
        private Action<string> labelFormatDelegate = null;
        private void SetLabelFormat(string value)
        {
            if (TAxis.Horizontal)
            {
                TAxis.Labels.DateTimeFormat = value;
            }
            else
            {
                TAxis.Labels.ValueFormat = value;
            }
        }

        public bool AutomaticMinMax
        {
            get { return TAxis.Automatic; }
            set
            {
                if (automaticMinMaxDelegate == null)
                    automaticMinMaxDelegate = new Action<bool>(SetAutomaticMinMax);
                chart.InvokeDelegate(automaticMinMaxDelegate, value);
            }
        }
        private Action<bool> automaticMinMaxDelegate = null;
        private void SetAutomaticMinMax(bool value)
        {
            TAxis.Automatic = value;
            if (!chart.SuspendUpdates)
                chart.ForceUpdate();
        }

        public bool AutomaticMinimum
        {
            get { return TAxis.AutomaticMinimum; }
            set
            {
                if (automaticMinimumDelegate == null)
                    automaticMinimumDelegate = new Action<bool>(SetAutomaticMinimum);
                chart.InvokeDelegate(automaticMinimumDelegate, value);
            }
        }
        private Action<bool> automaticMinimumDelegate = null;
        private void SetAutomaticMinimum(bool value)
        {
            TAxis.AutomaticMinimum = value;
            if (!chart.SuspendUpdates)
                chart.ForceUpdate();
        }

        public double Minimum
        {
            get { return TAxis.Minimum; }
            set
            {
                AutomaticMinimum = false;
                if (minimumDelegate == null)
                    minimumDelegate = new Action<double>(SetMinimum);
                chart.InvokeDelegate(minimumDelegate, value);
            }
        }
        private Action<double> minimumDelegate = null;
        private void SetMinimum(double value)
        {
            TAxis.Minimum = value;
            if (!chart.SuspendUpdates)
                chart.ForceUpdate();
        }

        public DateTime MinimumDateTime
        {
            get { return DateTime.FromOADate(Minimum); }
            set { Minimum = value.ToOADate(); }
        }

        public bool AutomaticMaximum
        {
            get { return TAxis.AutomaticMaximum; }
            set
            {
                if (automaticMaximumDelegate == null)
                    automaticMaximumDelegate = new Action<bool>(SetAutomaticMaximum);
                chart.InvokeDelegate(automaticMaximumDelegate, value);
            }
        }
        private Action<bool> automaticMaximumDelegate = null;
        private void SetAutomaticMaximum(bool value)
        {
            TAxis.AutomaticMaximum = value;
            if (!chart.SuspendUpdates)
                chart.ForceUpdate();
        }

        public double Maximum
        {
            get { return TAxis.Maximum; }
            set
            {
                AutomaticMaximum = false;
                if (maximumDelegate == null)
                    maximumDelegate = new Action<double>(SetMaximum);
                chart.InvokeDelegate(maximumDelegate, value);
            }
        }
        private Action<double> maximumDelegate = null;
        private void SetMaximum(double value)
        {
            TAxis.Maximum = value;
            if (!chart.SuspendUpdates)
                chart.ForceUpdate();
        }

        public DateTime MaximumDateTime
        {
            get { return DateTime.FromOADate(Maximum); }
            set { Maximum = value.ToOADate(); }
        }

        public double MinimumValue
        {
            get
            {
                if (IsHorizontal)
                    return TAxis.MinXValue;
                else
                    return TAxis.MinYValue;
            }
        }

        public double MaximumValue
        {
            get
            {
                if (IsHorizontal)
                    return TAxis.MaxXValue;
                else
                    return TAxis.MaxYValue;
            }
        }

        public int Offset
        {
            get { return TAxis.MaximumOffset; }
            set
            {
                if (offsetDelegate == null)
                    offsetDelegate = new Action<int>(SetOffset);
                chart.InvokeDelegate(offsetDelegate, value);
            }
        }
        private Action<int> offsetDelegate = null;
        private void SetOffset(int value)
        {
            if (value > 0)
            {
                TAxis.Automatic = false;
            }
            else
            {
                TAxis.Automatic = true;
            }
            TAxis.MaximumOffset = value;
            TAxis.MinimumOffset = value;
            if (!chart.SuspendUpdates)
                chart.ForceUpdate();
        }

        public bool Inverted
        {
            get { return TAxis.Inverted; }
            set
            {
                if (invertedDelegate == null)
                    invertedDelegate = new Action<bool>(SetInverted);
                chart.InvokeDelegate(invertedDelegate, value);
            }
        }
        private Action<bool> invertedDelegate = null;
        private void SetInverted(bool value)
        {
            TAxis.Inverted = value;
        }

        public bool Logarithmic
        {
            get { return TAxis.Logarithmic; }
            set
            {
                if (logarithmicDelegate == null)
                    logarithmicDelegate = new Action<bool>(SetLogarithmic);
                chart.InvokeDelegate(logarithmicDelegate, value);
            }
        }
        private Action<bool> logarithmicDelegate = null;
        private void SetLogarithmic(bool value)
        {
            TAxis.Logarithmic = value;
        }

        public double Increment
        {
            get { return TAxis.Increment; }
            set
            {
                if (incrementDelegate == null)
                    incrementDelegate = new Action<double>(SetIncrement);
                chart.InvokeDelegate(incrementDelegate, value);
            }
        }
        private Action<double> incrementDelegate = null;
        private void SetIncrement(double value)
        {
            TAxis.Increment = value;
        }

        public class AxisGrid
        {
            private Axis parent = null;
            private bool isMinor = false;
            internal AxisGrid(Axis _parent) : this(_parent, false) { }
            internal AxisGrid(Axis _parent, bool _isMinor)
            {
                parent = _parent;
                isMinor = _isMinor;
            }

            public bool Visible
            {
                get
                {
                    if (isMinor)
                        return parent.TAxis.MinorGrid.Visible;
                    else
                        return parent.TAxis.Grid.Visible;
                }
                set
                {
                    if (visibleDelegate == null)
                        visibleDelegate = new Action<bool>(SetVisible);
                    parent.chart.InvokeDelegate(visibleDelegate, value);
                }
            }
            private Action<bool> visibleDelegate = null;
            private void SetVisible(bool value)
            {
                if (isMinor)
                    parent.TAxis.MinorGrid.Visible = value;
                else
                    parent.TAxis.Grid.Visible = value;
            }

            public System.Drawing.Drawing2D.DashStyle Style
            {
                get
                {
                    if (isMinor)
                        return parent.TAxis.MinorGrid.Style;
                    else
                        return parent.TAxis.Grid.Style;
                }
                set
                {
                    if (styleDelegate == null)
                        styleDelegate = new Action<System.Drawing.Drawing2D.DashStyle>(SetStyle);
                    parent.chart.InvokeDelegate(styleDelegate, value);
                }
            }
            private Action<System.Drawing.Drawing2D.DashStyle> styleDelegate = null;
            private void SetStyle(System.Drawing.Drawing2D.DashStyle value)
            {
                if (isMinor)
                    parent.TAxis.MinorGrid.Style = value;
                else
                    parent.TAxis.Grid.Style = value;
            }
        }
        private AxisGrid majorGrid = null;
        public AxisGrid MajorGrid
        {
            get
            {
                if (majorGrid == null)
                    majorGrid = new AxisGrid(this);
                return majorGrid;
            }
        }
        private AxisGrid minorGrid = null;
        public AxisGrid MinorGrid
        {
            get
            {
                if (minorGrid == null)
                    minorGrid = new AxisGrid(this, true);
                return minorGrid;
            }
        }

        public class AxisTicks
        {
            protected Axis parent = null;
            protected Steema.TeeChart.Axis.TicksPen tTicks = null;
            internal AxisTicks(Axis _parent, Steema.TeeChart.Axis.TicksPen _tTicks)
            {
                parent = _parent;
                tTicks = _tTicks;
            }

            public bool Visible
            {
                get { return tTicks.Visible; }
                set
                {
                    if (visibleDelegate == null)
                        visibleDelegate = new Action<bool>(SetVisible);
                    parent.chart.InvokeDelegate(visibleDelegate, value);
                }
            }
            private Action<bool> visibleDelegate = null;
            private void SetVisible(bool value)
            {
                tTicks.Visible = value;
            }

            public Color Color
            {
                get { return tTicks.Color; }
                set
                {
                    if (colorDelegate == null)
                        colorDelegate = new Action<Color>(SetColor);
                    parent.chart.InvokeDelegate(colorDelegate, value);
                }
            }
            private Action<Color> colorDelegate = null;
            private void SetColor(Color value)
            {
                tTicks.Color = value;
            }

            public int Width
            {
                get { return tTicks.Width; }
                set
                {
                    if (widthDelegate == null)
                        widthDelegate = new Action<int>(SetWidth);
                    parent.chart.InvokeDelegate(widthDelegate, value);
                }
            }
            private Action<int> widthDelegate = null;
            private void SetWidth(int value)
            {
                tTicks.Width = value;
            }
        }
        public class AxisMinorTicks : AxisTicks
        {
            internal AxisMinorTicks(Axis _parent, Steema.TeeChart.Axis.TicksPen _tTicks) : base(_parent, _tTicks) { }

            public int Count
            {
                get { return parent.TAxis.MinorTickCount; }
                set
                {
                    if (countDelegate == null)
                        countDelegate = new Action<int>(SetCount);
                    parent.chart.InvokeDelegate(countDelegate, value);
                }
            }
            private Action<int> countDelegate = null;
            private void SetCount(int value)
            {
                parent.TAxis.MinorTickCount = value;
            }
        }
        private AxisTicks majorTicks = null;
        public AxisTicks MajorTicks
        {
            get
            {
                if (majorTicks == null)
                    majorTicks = new AxisTicks(this, TAxis.Ticks);
                return majorTicks;
            }
        }
        private AxisMinorTicks minorTicks = null;
        public AxisMinorTicks MinorTicks
        {
            get
            {
                if (minorTicks == null)
                    minorTicks = new AxisMinorTicks(this, TAxis.MinorTicks);
                return minorTicks;
            }
        }

        public double PixelCoordinateToValue(Point p)
        {
            return PixelCoordinateToValue(p.X, p.Y);
        }
        /// <summary>
        /// This method will find a specific pixel coordinate (relative to the Chart control) and return the value at that point for this axis.
        /// This particular overload will auto-detect whether this is a horizontal or vertical axis, so only takes one parameter (x-coordinate for horizontal, y-coordinate for vertical).
        /// </summary>
        /// <param name="xy"></param>
        /// <returns></returns>
        public double PixelCoordinateToValue(int xy)
        {
            return PixelCoordinateToValue(xy, xy);
        }
        public double PixelCoordinateToValue(int x, int y)
        {
            if (TAxis.Horizontal)
            {
                return TAxis.CalcPosPoint(x);
            }
            else
            {
                return TAxis.CalcPosPoint(y);
            }
        }

        public int ValueToPixelCoordinate(double value)
        {
            if (TAxis.Horizontal)
            {
                return TAxis.CalcXPosValue(value);
            }
            else
            {
                return TAxis.CalcYPosValue(value);
            }
        }

        public int Width
        {
            get
            {
                return Convert.ToInt32(Math.Ceiling((double)TAxis.AxisPen.Width + (double)TAxis.Labels.LabelWidth(100) + (double)TAxis.MinorTicks.Length + 35.0));
            }
        }
        internal void Reposition(int position)
        {
            if (repositionDelegate == null)
                repositionDelegate = new Action<int>(RepositionInternal);
            chart.InvokeDelegate(repositionDelegate, position);
        }
        private Action<int> repositionDelegate = null;
        private void RepositionInternal(int position)
        {
            TAxis.RelativePosition = position * -70; //CKBN New TeeChart Was '* -60'
        }

        internal Axis(Chart _chart, string unit, bool horizontal = false)
        {
            chart = _chart;
            TAxis = new Steema.TeeChart.Axis(horizontal, false, chart.TChart.Chart);
            Unit = unit;

            if (!horizontal)
            {
                TAxis.Title.Angle = 90;
                TAxis.PositionUnits = Steema.TeeChart.PositionUnits.Pixels;
                Reposition(chart.VerticalAxes.Count);
            }
        }

        internal void Remove()
        {
            if (removeDelegate == null)
                removeDelegate = new Action(RemoveInternal);
            chart.InvokeDelegate(removeDelegate);
        }
        private Action removeDelegate = null;
        private void RemoveInternal()
        {
            chart.TChart.Axes.Custom.Remove(TAxis);
        }
    }
}