﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace Ellab.Charting
{
    public class ColorLine : ITool
    {
        private Steema.TeeChart.Tools.ColorLine tColorLine;
        private Axis axis;

        public event EventHandler ValueChanged;
        public event EventHandler<ColorLineEventArgs> DragLine;
        public event EventHandler EndDragLine;

        public Chart Chart
        {
            get;
            private set;
        }

        public ToolType Type { get { return ToolType.ColorLine; } }

        public string Name
        {
            get;
            set;
        }

        public Axis Axis
        {
            get { return axis; }
            set
            {
                axis = value;
                if(axisDelegate == null)
                    axisDelegate = new Action<Steema.TeeChart.Axis>(SetAxis);
                Chart.InvokeDelegate(axisDelegate, axis.TAxis);
            }
        }
        private Action<Steema.TeeChart.Axis> axisDelegate = null;
        private void SetAxis(Steema.TeeChart.Axis value)
        {
            tColorLine.Axis = value;
        }

        public string Unit
        {
            get { return axis.Unit; }
            set { Axis = Chart.GetAxis(value); }
        }

        public double Value
        {
            get { return tColorLine.Value; }
            set
            {
                if (valueDelegate == null)
                    valueDelegate = new Action<double>(SetValue);
                Chart.InvokeDelegate(valueDelegate, value);
            }
        }
        private Action<double> valueDelegate = null;
        private void SetValue(double value)
        {
            tColorLine.Value = value;
            if (ValueChanged != null)
                ValueChanged(this, new EventArgs());
        }

        public DateTime DateTimeValue
        {
            get { return DateTime.FromOADate(Value); }
            set { Value = value.ToOADate(); }
        }

        public Color Color
        {
            get { return tColorLine.Pen.Color; }
            set
            {
                if (colorDelegate == null)
                    colorDelegate = new Action<Color>(SetColor);
                Chart.InvokeDelegate(colorDelegate, value);
            }
        }
        private Action<Color> colorDelegate = null;
        private void SetColor(Color value)
        {
            tColorLine.Pen.Color = value;
        }

        public DashStyle Style
        {
            get
            {
                return tColorLine.Pen.Style;
            }
            set
            {
                if (styleDelegate == null)
                    styleDelegate = new Action<DashStyle>(SetStyle);
                Chart.InvokeDelegate(styleDelegate, value);
            }
        }
        private Action<DashStyle> styleDelegate = null;
        private void SetStyle(DashStyle value)
        {
            tColorLine.Pen.Style = value;
        }

        public int Width
        {
            get { return tColorLine.Pen.Width; }
            set
            {
                if (widthDelegate == null)
                    widthDelegate = new Action<int>(SetWidth);
                Chart.InvokeDelegate(widthDelegate, value);
            }
        }
        private Action<int> widthDelegate = null;
        private void SetWidth(int value)
        {
            tColorLine.Pen.Width = value;
        }

        public bool AllowMouseDrag
        {
            get { return tColorLine.AllowDrag; }
            set
            {
                if (allowMouseDragDelegate == null)
                    allowMouseDragDelegate = new Action<bool>(SetAllowMouseDrag);
                Chart.InvokeDelegate(allowMouseDragDelegate, value);
            }
        }
        private Action<bool> allowMouseDragDelegate = null;
        private void SetAllowMouseDrag(bool value)
        {
            tColorLine.AllowDrag = value;
        }

        public bool AllowMouseDragOutsideChart
        {
            get { return tColorLine.NoLimitDrag; }
            set
            {
                if (allowMouseDragOutsideChartDelegate == null)
                    allowMouseDragOutsideChartDelegate = new Action<bool>(SetAllowMouseDragOutsideChart);
                Chart.InvokeDelegate(allowMouseDragOutsideChartDelegate, value);
            }
        }
        private Action<bool> allowMouseDragOutsideChartDelegate = null;
        private void SetAllowMouseDragOutsideChart(bool value)
        {
            tColorLine.NoLimitDrag = value;
        }

        private ColorLine(Chart _chart, string name, Axis _axis, double _value, Color _color)
        {
            Chart = _chart;
            Name = name;
            tColorLine = new Steema.TeeChart.Tools.ColorLine(Chart.TChart.Chart);
            Axis = _axis;
            Color = _color;
            Value = _value;
            Width = 2;

            tColorLine.AllowDrag = false;
            tColorLine.DragRepaint = true; //CKBN New TeeChart. This might cause trouble in large sessions. I've set it to true, to compensate for blur colorline when draging

            this.tColorLine.DragLine += new EventHandler(tColorLine_DragLine);
            this.tColorLine.EndDragLine += new Steema.TeeChart.Tools.ColorLineToolOnDragEventHandler(tColorLine_EndDragLine);

            Chart.AddTool(this);
        }

        public static ColorLine Create(Chart _chart, string name, string unit, DateTime _value, Color _color)
        {
            return ColorLine.Create(_chart, name, _chart.GetAxis(unit), _value.ToOADate(), _color);
        }
        public static ColorLine Create(Chart _chart, string name, Axis _axis, DateTime _value, Color _color)
        {
            return ColorLine.Create(_chart, name, _axis, _value.ToOADate(), _color);
        }
        public static ColorLine Create(Chart _chart, string name, string unit, double _value, Color _color)
        {
            return ColorLine.Create(_chart, name, _chart.GetAxis(unit), _value, _color);
        }
        public static ColorLine Create(Chart _chart, string name, Axis _axis, double _value, Color _color)
        {
            return new ColorLine(_chart, name, _axis, _value, _color);
        }

        /// <summary>
        /// Returns whether the series travels through the given pixel-coordinate.
        /// </summary>
        /// <param name="p"></param>
        /// <returns></returns>
        public bool IntersectsWith(Point p)
        {
            return IntersectsWith(p.X, p.Y);
        }
        /// <summary>
        /// Returns whether the series travels through the given pixel-coordinate.
        /// </summary>
        /// <param name="p"></param>
        /// <returns></returns>
        public bool IntersectsWith(int x, int y)
        {
            return tColorLine.Clicked(x, y);
        }

        public ITool CreateMirror(Chart _chart)
        {
            ColorLine c = ColorLine.Create(_chart, Name, Unit, Value, Color);
            c.Width = this.Width;
            this.ValueChanged += delegate(object sender, EventArgs e)
            {
                c.Value = this.Value;
            };

            return c;
        }

        void tColorLine_EndDragLine(object sender)
        {
            try
            {
                if (this.EndDragLine != null)
                {
                    EndDragLine(sender, new EventArgs());
                }
            }
            catch
            { 
            }
        }

        void tColorLine_DragLine(object sender, EventArgs e)
        {
            try
            {
                if (this.DragLine != null)
                {
                    ColorLineEventArgs colorLineEventArgs = new ColorLineEventArgs((sender as Steema.TeeChart.Tools.ColorLine).Value);
                    DragLine(this, colorLineEventArgs);
                }
            }
            catch
            { 
            }
        }

        public void Remove()
        {
            if (removeDelegate == null)
                removeDelegate = new Action(RemoveInternal);
            Chart.InvokeDelegate(removeDelegate);
        }
        private Action removeDelegate = null;
        private void RemoveInternal()
        {
            Chart.TChart.Tools.Remove(tColorLine);
            Chart.RemoveTool(this);
        }
    }
}
