﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace Ellab.Charting
{
    public interface ISeries
    {
        Chart Chart { get; }

        bool XValuesAsDataTime { set; }

        void Remove();
        void RemoveAll();

        string Unit { get; set; }

        SeriesType Type { get; }

        string Name { get; set; }

        bool Visible { get; set; }

        Color Color { get; set; }

        //int Width { get; set; }

        ISeriesData Data { get; }

        Axis HorizontalAxis { get; }

        Axis VerticalAxis { get; set; }

        Charting.MarksTip MarksTip { get; set; }

        Charting.NearestPoint NearestPoint { get; set; }

        bool IntersectsWith(Point p);
        bool IntersectsWith(int x, int y);
        int SectionIntersectingWith(Point p);
        int SectionIntersectingWith(int x, int y);

        ISeries CreateMirror(Chart chart);

        void RefreshDataSource();

        Steema.TeeChart.Styles.Series baseSeries { get; }
    }

    public interface ISeriesData
    {
        int Count { get; }

        Coordinate this[int index] { get; }

        void SetX(int index, double xValue);
        void SetY(int index, double yValue);

        /// <summary>
        /// Returns the Y-value at the given X-value.
        /// This will never attempt to interpolate a value! If the exact X-value given is not present, null is returned.
        /// </summary>
        /// <param name="xValue"></param>
        /// <returns></returns>
        double? this[DateTime xValue] { get; set; }

        void Add(double x, double y);
        void Add(DateTime x, double y);
        void Add(double[] x, double[] y);

        void Remove(DateTime xValue);
        void Remove(int index);

        void Clear();

        /// <summary>
        /// Attempts to interpolate the Y-value at the given X-value.
        /// Interpolation must occur between two existing points and so cannot be done before the first or after the last X-value. Attempting to do so will return null.
        /// </summary>
        /// <param name="xValue"></param>
        /// <returns></returns>
        double? InterpolateValue(DateTime xValue);
        /// <summary>
        /// Attempts to interpolate the Y-value at the given X-value.
        /// Interpolation must occur between two existing points and so cannot be done before the first or after the last X-value. Attempting to do so will return null.
        /// </summary>
        /// <param name="xValue"></param>
        /// <returns></returns>
        double? InterpolateValue(double xValue);

        int? IndexOf(DateTime xValue);
        int? IndexOf(double xValue);

        DateTime MinimumX { get; }
        DateTime MaximumX { get; }
        double MinimumY { get; }
        double MaximumY { get; }
    }

    public struct Coordinate
    {
        public double X;
        public double Y;
        public Coordinate(double x, double y) { X = x; Y = y; }
    }
}
