﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace Ellab.Charting
{
    public class LineSeries : LineSeriesBase
    {
        protected Steema.TeeChart.Styles.Line tSeriesAsLine
        {
            get { return (Steema.TeeChart.Styles.Line)tSeries; }
        }
        protected Steema.TeeChart.Styles.FastLine tSeriesAsFastLine
        {
            get { return (Steema.TeeChart.Styles.FastLine)tSeries; }
        }

        #region Events

        public event System.Windows.Forms.MouseEventHandler Click;

        #endregion Events

        #region Outline

        public class LineSeriesOutline
        {
            private LineSeries parent = null;
            internal LineSeriesOutline(LineSeries _parent)
            {
                parent = _parent;
            }

            public bool Visible
            {
                get
                {
                    if (!parent.IsFastLine)
                        return parent.tSeriesAsLine.OutLine.Visible;
                    return false;
                }
                set
                {
                    if (visibleDelegate == null)
                        visibleDelegate = new Action<bool>(SetVisible);
                    parent.Chart.InvokeDelegate(visibleDelegate, value);
                }
            }
            private Action<bool> visibleDelegate = null;
            private void SetVisible(bool value)
            {
                if (!parent.IsFastLine)
                    parent.tSeriesAsLine.OutLine.Visible = value;
            }

            public int Width
            {
                get
                {
                    if (!parent.IsFastLine)
                        return parent.tSeriesAsLine.OutLine.Width;
                    return 0;
                }
                set
                {
                    if (widthDelegate == null)
                        widthDelegate = new Action<int>(SetWidth);
                    parent.Chart.InvokeDelegate(widthDelegate, value);
                }
            }
            private Action<int> widthDelegate = null;
            private void SetWidth(int value)
            {
                if (!parent.IsFastLine)
                    parent.tSeriesAsLine.OutLine.Width = value;
            }

            public System.Drawing.Color Color
            {
                get
                {
                    if (!parent.IsFastLine)
                        return parent.tSeriesAsLine.OutLine.Color;
                    else
                        return Color.Transparent;
                }
                set
                {
                    if (colorDelegate == null)
                        colorDelegate = new Action<System.Drawing.Color>(SetColor);
                    parent.Chart.InvokeDelegate(colorDelegate, value);
                }
            }
            private Action<System.Drawing.Color> colorDelegate = null;
            private void SetColor(System.Drawing.Color value)
            {
                if (!parent.IsFastLine)
                    parent.tSeriesAsLine.OutLine.Color = value;
            }
        }
        private LineSeriesOutline outline = null;
        public LineSeriesOutline Outline
        {
            get
            {
                if (outline == null)
                    outline = new LineSeriesOutline(this);
                return outline;
            }
        }

        #endregion Outline

        #region Marker

        public class LineSeriesMarker
        {
            private LineSeries parent = null;
            internal LineSeriesMarker(LineSeries _parent)
            {
                parent = _parent;
            }

            public bool Visible
            {
                get
                {
                    if (!parent.IsFastLine)
                        return parent.tSeriesAsLine.Pointer.Visible;
                    else
                        return false;
                }
                set
                {
                    if (visibleDelegate == null)
                        visibleDelegate = new Action<bool>(SetVisible);
                    parent.Chart.InvokeDelegate(visibleDelegate, value);
                }
            }
            private Action<bool> visibleDelegate = null;
            private void SetVisible(bool value)
            {
                if (!parent.IsFastLine)
                    parent.tSeriesAsLine.Pointer.Visible = value;
            }

            public Color Color
            {
                get
                {
                    if (!parent.IsFastLine)
                        return parent.tSeriesAsLine.Pointer.Color;
                    else
                        return Color.Transparent;
                }
                set
                {
                    if (colorDelegate == null)
                        colorDelegate = new Action<Color>(SetColor);
                    parent.Chart.InvokeDelegate(colorDelegate, value);
                }
            }
            private Action<Color> colorDelegate = null;
            private void SetColor(Color value)
            {
                if (!parent.IsFastLine)
                    parent.tSeriesAsLine.Pointer.Color = value;
            }

            public int Size
            {
                get
                {
                    if (!parent.IsFastLine)
                        return parent.tSeriesAsLine.Pointer.VertSize;
                    else
                        return 0;
                }
                set
                {
                    if (sizeDelegate == null)
                        sizeDelegate = new Action<int>(SetSize);
                    parent.Chart.InvokeDelegate(sizeDelegate, value);
                }
            }
            private Action<int> sizeDelegate = null;
            private void SetSize(int value)
            {
                if (!parent.IsFastLine)
                {
                    parent.tSeriesAsLine.Pointer.VertSize = value;
                    parent.tSeriesAsLine.Pointer.HorizSize = value;
                }
            }

            #region Pointer Style
            public MarkerStyle Style
            {
                get
                {
                    if (!parent.IsFastLine)
                        return TeeChart2MarkerStyle(parent.tSeriesAsLine.Pointer.Style);
                    else
                        return MarkerStyle.Rectangle;
                }
                set
                {
                    if (styleDelegate == null)
                        styleDelegate = new Action<MarkerStyle>(SetStyle);
                    parent.Chart.InvokeDelegate(styleDelegate, value);
                }
            }
            private Action<MarkerStyle> styleDelegate = null;
            private void SetStyle(MarkerStyle value)
            {
                if (!parent.IsFastLine)
                    parent.tSeriesAsLine.Pointer.Style = MarkerStyle2TeeChart(value);
            }
            public enum MarkerStyle
            {
                Rectangle, Circle, Sphere, Cross
            }   //Remember to add conversions to TeeChart2MarkerStyle and MarkerStyle2TeeChart methods when adding to the enum!
            private MarkerStyle TeeChart2MarkerStyle(Steema.TeeChart.Styles.PointerStyles tcStyle)
            {
                switch (tcStyle)
                {
                    case Steema.TeeChart.Styles.PointerStyles.Circle: return MarkerStyle.Circle;
                    case Steema.TeeChart.Styles.PointerStyles.Rectangle: return MarkerStyle.Rectangle;
                    case Steema.TeeChart.Styles.PointerStyles.Sphere: return MarkerStyle.Sphere;
                    case Steema.TeeChart.Styles.PointerStyles.Cross: return MarkerStyle.Cross;
                    default: return MarkerStyle.Rectangle;
                }
            }
            private Steema.TeeChart.Styles.PointerStyles MarkerStyle2TeeChart(MarkerStyle style)
            {
                switch (style)
                {
                    case MarkerStyle.Circle: return Steema.TeeChart.Styles.PointerStyles.Circle;
                    case MarkerStyle.Cross: return Steema.TeeChart.Styles.PointerStyles.Cross;
                    case MarkerStyle.Rectangle: return Steema.TeeChart.Styles.PointerStyles.Rectangle;
                    case MarkerStyle.Sphere: return Steema.TeeChart.Styles.PointerStyles.Sphere;
                    default: return Steema.TeeChart.Styles.PointerStyles.Rectangle;
                }
            }
            #endregion
        }
        private LineSeriesMarker marker = null;
        public LineSeriesMarker Marker
        {
            get
            {
                if (marker == null)
                    marker = new LineSeriesMarker(this);
                return marker;
            }
        }

        #endregion Marker

        #region FastLine

        public bool IsFastLine
        {
            get
            {
                if (tSeries is Steema.TeeChart.Styles.FastLine)
                    return true;
                else
                    return false;
            }
            set
            {
                if (fastLineDelegate == null)
                    fastLineDelegate = new Action<bool>(SetFastLine);
                Chart.InvokeDelegate(fastLineDelegate, value);
            }
        }
        private Action<bool> fastLineDelegate = null;
        private void SetFastLine(bool value)
        {
            if (value != IsFastLine)
            {
                System.Type type = null;
                if (value == true)
                    type = typeof(Steema.TeeChart.Styles.FastLine);
                else
                    type = typeof(Steema.TeeChart.Styles.Line);

                int width = Width;

                Steema.TeeChart.Styles.Series.ChangeType(ref tSeries, type);

                Width = width;
            }
        }


        public bool DrawAllPoints
        {
            get
            {
                if (IsFastLine)
                    return tSeriesAsFastLine.DrawAllPoints;
                else
                    return true;
            }
            set
            {
                if (drawAllPointsDelegate == null)
                    drawAllPointsDelegate = new Action<bool>(SetDrawAllPoints);
                Chart.InvokeDelegate(drawAllPointsDelegate, value);
            }
        }
        private Action<bool> drawAllPointsDelegate = null;
        private void SetDrawAllPoints(bool value)
        {
            if (IsFastLine)
                tSeriesAsFastLine.DrawAllPoints = value;
        }

        #endregion FastLine

        #region Construction

        private LineSeries(Chart _chart, string name, string unit)
        {
            Unit = unit;

            tSeries = new Steema.TeeChart.Styles.Line(_chart.TChart.Chart);
            Chart = _chart;

            Name = name;

            //TeeChart Initialization
            tSeries.CustomHorizAxis = HorizontalAxis.TAxis;
            tSeries.CustomVertAxis = VerticalAxis.TAxis;

            //Default values, optional
            tSeriesAsLine.Pointer.Style = Steema.TeeChart.Styles.PointerStyles.Rectangle;
            tSeriesAsLine.Pointer.Visible = true;

            Width = 1;
            //tSeriesAsLine.OutLine.Width = 1;
            //tSeriesAsLine.OutLine.Color = System.Drawing.Color.Black;
            tSeriesAsLine.OutLine.Visible = false;

            //Events
            tSeries.Click += delegate(object sender, System.Windows.Forms.MouseEventArgs e)
            {
                if (Click != null)
                    Click(this, e);
            };

            base.MarksTip = MarksTip.Create(Chart, name + " MarksTip", this);
            base.NearestPoint = NearestPoint.Create(Chart, name + " NearestPoint", this);
        }

        public static LineSeries Create(Chart _chart, string name, string unit)
        {
            LineSeries s = new LineSeries(_chart, name, unit);
            _chart.AddSeries(s);
            return s;
        }

        public override ISeries CreateMirror(Chart _chart)
        {
            LineSeries s = new LineSeries(_chart, Name, Unit);
            s.Color = this.Color;
            s.Marker.Visible = this.Marker.Visible;
            s.Outline.Color = this.Outline.Color;
            s.Outline.Visible = this.Outline.Visible;
            s.Outline.Width = this.Outline.Width;
            s.Width = this.Width;

            this.PointAdded += delegate(object sender, EventArgs e)
            {
                s.RefreshDataSource();
            };
            this.PointRemoved += delegate(object sender, EventArgs e)
            {
                s.RefreshDataSource();
            };

            s.tSeries.DataSource = tSeries;

            _chart.AddSeries(s);
            return s;
        }

        #endregion Construction
    }
}