﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace Ellab.Charting
{
    public class PointSeries : SeriesBase
    {
        protected Steema.TeeChart.Styles.Points TSeries
        {
            get { return (Steema.TeeChart.Styles.Points)tSeries; }
            private set { tSeries = value; }
        }

        #region Events

        public event System.Windows.Forms.MouseEventHandler Click;

        #endregion Events

        #region Marker

        public class PointSeriesMarker
        {
            private PointSeries parent = null;
            internal PointSeriesMarker(PointSeries _parent)
            {
                parent = _parent;
            }

            public bool Visible
            {
                get { return parent.TSeries.Pointer.Visible; }
                set
                {
                    if (visibleDelegate == null)
                        visibleDelegate = new Action<bool>(SetVisible);
                    parent.Chart.InvokeDelegate(visibleDelegate, value);
                }
            }
            private Action<bool> visibleDelegate = null;
            private void SetVisible(bool value)
            {
                parent.TSeries.Pointer.Visible = value;
            }

            public Color Color
            {
                get { return parent.TSeries.Pointer.Color; }
                set
                {
                    if (colorDelegate == null)
                        colorDelegate = new Action<Color>(SetColor);
                    parent.Chart.InvokeDelegate(colorDelegate, value);
                }
            }
            private Action<Color> colorDelegate = null;
            private void SetColor(Color value)
            {
                parent.TSeries.Pointer.Color = value;
            }

            public int Size
            {
                get { return parent.TSeries.Pointer.VertSize; }
                set
                {
                    if (sizeDelegate == null)
                        sizeDelegate = new Action<int>(SetSize);
                    parent.Chart.InvokeDelegate(sizeDelegate, value);
                }
            }
            private Action<int> sizeDelegate = null;
            private void SetSize(int value)
            {
                parent.TSeries.Pointer.VertSize = value;
                parent.TSeries.Pointer.HorizSize = value;
            }

            #region Pointer Style
            public MarkerStyle Style
            {
                get
                {
                    return TeeChart2MarkerStyle(parent.TSeries.Pointer.Style);
                }
                set
                {
                    if (styleDelegate == null)
                        styleDelegate = new Action<MarkerStyle>(SetStyle);
                    parent.Chart.InvokeDelegate(styleDelegate, value);
                }
            }
            private Action<MarkerStyle> styleDelegate = null;
            private void SetStyle(MarkerStyle value)
            {
                parent.TSeries.Pointer.Style = MarkerStyle2TeeChart(value);
            }
            public enum MarkerStyle
            {
                Rectangle, Circle, Sphere, Cross
            }   //Remember to add conversions to TeeChart2MarkerStyle and MarkerStyle2TeeChart methods when adding to the enum!
            private MarkerStyle TeeChart2MarkerStyle(Steema.TeeChart.Styles.PointerStyles tcStyle)
            {
                switch (tcStyle)
                {
                    case Steema.TeeChart.Styles.PointerStyles.Circle: return MarkerStyle.Circle;
                    case Steema.TeeChart.Styles.PointerStyles.Rectangle: return MarkerStyle.Rectangle;
                    case Steema.TeeChart.Styles.PointerStyles.Sphere: return MarkerStyle.Sphere;
                    case Steema.TeeChart.Styles.PointerStyles.Cross: return MarkerStyle.Cross;
                    default: return MarkerStyle.Rectangle;
                }
            }
            private Steema.TeeChart.Styles.PointerStyles MarkerStyle2TeeChart(MarkerStyle style)
            {
                switch (style)
                {
                    case MarkerStyle.Circle: return Steema.TeeChart.Styles.PointerStyles.Circle;
                    case MarkerStyle.Cross: return Steema.TeeChart.Styles.PointerStyles.Cross;
                    case MarkerStyle.Rectangle: return Steema.TeeChart.Styles.PointerStyles.Rectangle;
                    case MarkerStyle.Sphere: return Steema.TeeChart.Styles.PointerStyles.Sphere;
                    default: return Steema.TeeChart.Styles.PointerStyles.Rectangle;
                }
            }
            #endregion
        }
        private PointSeriesMarker marker = null;
        public PointSeriesMarker Marker
        {
            get
            {
                if(marker == null)
                    marker = new PointSeriesMarker(this);
                return marker;
            }
        }

        #endregion Marker

        #region Construction

        private PointSeries(Chart _chart, string name, string unit)
        {
            Unit = unit;

            TSeries = new Steema.TeeChart.Styles.Points(_chart.TChart.Chart);
            Chart = _chart;

            Name = name;

            //TeeChart Initialization
            tSeries.CustomHorizAxis = HorizontalAxis.TAxis;
            tSeries.CustomVertAxis = VerticalAxis.TAxis;

            //Default values, optional
            TSeries.Pointer.Style = Steema.TeeChart.Styles.PointerStyles.Rectangle;
            TSeries.Pointer.Visible = true;

            //Events
            TSeries.Click += delegate(object sender, System.Windows.Forms.MouseEventArgs e)
            {
                if (Click != null)
                    Click(this, e);
                int blah = TSeries.Clicked(e.X + 20, e.Y);
                if (e.Button == System.Windows.Forms.MouseButtons.Right)
                    System.Windows.Forms.MessageBox.Show("Højreklikhurra! " + blah);
            };
        }

        public static PointSeries Create(Chart _chart, string name, string unit)
        {
            PointSeries s = new PointSeries(_chart, name, unit);
            _chart.AddSeries(s);
            return s;
        }

        public override ISeries CreateMirror(Chart _chart)
        {
            PointSeries s = new PointSeries(_chart, Name, Unit);
            s.Color = this.Color;
            s.Marker.Visible = this.Marker.Visible;

            this.PointAdded += delegate(object sender, EventArgs e)
            {
                s.RefreshDataSource();
            };
            this.PointRemoved += delegate(object sender, EventArgs e)
            {
                s.RefreshDataSource();
            };

            s.tSeries.DataSource = tSeries;

            _chart.AddSeries(s);
            return s;
        }

        #endregion Construction
    }
}