﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Ellab.Charting
{
    public class ShapeSeries : SeriesBase
    {
        private bool positionSet = false; //Has the user set the position of the series? If not, it should be kept invisible.
        private bool desiredVisible = true; //Hide the series until the user sets its position. Use this to check if the user wants the series to be visible after position is set.
        protected Steema.TeeChart.Styles.Shape TSeries
        {
            get { return (Steema.TeeChart.Styles.Shape)tSeries; }
            private set { tSeries = value; }
        }

        #region Events

        public event System.Windows.Forms.MouseEventHandler Click;

        #endregion Events

        public ShapeStyle Style
        {
            get { return TeeChart2ShapeStyle(TSeries.Style); }
            set
            {
                if (styleDelegate == null)
                    styleDelegate = new Action<ShapeStyle>(SetStyle);
                Chart.InvokeDelegate(styleDelegate, value);
            }
        }
        private Action<ShapeStyle> styleDelegate = null;
        private void SetStyle(ShapeStyle value)
        {
            TSeries.Style = ShapeStyle2TeeChart(value);
        }

        public Coordinate ValuePosition
        {
            get
            {
                return new Coordinate(ValuePositionX, ValuePositionY);
            }
            set
            {
                ValuePositionX = value.X;
                ValuePositionY = value.Y;
            }
        }

        public double ValuePositionX
        {
            get
            {
                if (!HorizontalAxis.Inverted)
                {
                    if (TSeries.X0 > TSeries.X1)
                        return TSeries.X0;
                    else
                        return TSeries.X1;
                }
                else
                {
                    if (TSeries.X0 < TSeries.X1)
                        return TSeries.X0;
                    else
                        return TSeries.X1;
                }
            }
            set
            {
                if (valuePositionXDelegate == null)
                    valuePositionXDelegate = new Action<double>(SetValuePositionX);
                Chart.InvokeDelegate(valuePositionXDelegate, value);
            }
        }
        private Action<double> valuePositionXDelegate = null;
        private void SetValuePositionX(double value)
        {
            double width = Math.Abs(TSeries.X0 - TSeries.X1);
            TSeries.X0 = value;
            TSeries.X1 = value + width;
            positionSet = true;
            Visible = desiredVisible;
        }

        public double ValuePositionY
        {
            get
            {
                if (!VerticalAxis.Inverted)
                {
                    if (TSeries.Y0 < TSeries.Y1)
                        return TSeries.Y0;
                    else
                        return TSeries.Y1;
                }
                else
                {
                    if (TSeries.Y0 > TSeries.Y1)
                        return TSeries.Y0;
                    else
                        return TSeries.Y1;
                }
            }
            set
            {
                if (valuePositionYDelegate == null)
                    valuePositionYDelegate = new Action<double>(SetValuePositionY);
                Chart.InvokeDelegate(valuePositionYDelegate, value);
            }
        }
        private Action<double> valuePositionYDelegate = null;
        private void SetValuePositionY(double value)
        {
            double height = Math.Abs(TSeries.Y0 - TSeries.Y1);
            TSeries.Y0 = value;
            TSeries.Y1 = value - height;    //Use the upper pixel coordinate for Y0. Therefore, Y1 will have the smaller value a
            positionSet = true;
            Visible = desiredVisible;
        }

        public double Width
        {
            get { return Math.Abs(TSeries.X0 - TSeries.X1); }
            set
            {
                if (widthDelegate == null)
                    widthDelegate = new Action<double>(SetWidth);
                Chart.InvokeDelegate(widthDelegate, value);
            }
        }
        private Action<double> widthDelegate = null;
        private void SetWidth(double value)
        {
            if (TSeries.X0 > TSeries.X1)    //Maintain X0 as the "smaller" value
                TSeries.X0 = TSeries.X1;
            TSeries.X1 = TSeries.X0 + value;
            positionSet = true;
            Visible = desiredVisible;
        }

        public TimeSpan WidthTimeSpan
        {
            get { return (DateTime.FromOADate(ValuePositionX + Width) - DateTime.FromOADate(ValuePositionX)); }
            set
            {
                Width = (DateTime.FromOADate(ValuePositionX) + value).ToOADate() - ValuePositionX;
            }
        }

        public double Height
        {
            get { return Math.Abs(TSeries.Y0 - TSeries.Y1); }
            set
            {
                if (heightDelegate == null)
                    heightDelegate = new Action<double>(SetHeight);
                Chart.InvokeDelegate(heightDelegate, value);
            }
        }
        private Action<double> heightDelegate = null;
        private void SetHeight(double value)
        {
            if (TSeries.Y0 < TSeries.Y1)    //Maintain Y0 as the "larger" value (upper pixel coordinate)
                TSeries.Y0 = TSeries.Y1;
            TSeries.Y1 = TSeries.Y0 - value;
            positionSet = true;
            Visible = desiredVisible;
        }

        public new bool Visible
        {
            get { return desiredVisible; }
            set
            {
                if (visibleDelegate == null)
                    visibleDelegate = new Action<bool>(SetVisible);
                if (positionSet)
                    Chart.InvokeDelegate(visibleDelegate, value);
                else
                    Chart.InvokeDelegate(visibleDelegate, false);
                desiredVisible = value;
            }
        }
        private Action<bool> visibleDelegate = null;
        private void SetVisible(bool value)
        {
            TSeries.Visible = value;
        }

        #region Construction

        private ShapeSeries(Chart _chart, string name, string unit)
        {
            Unit = unit;

            TSeries = new Steema.TeeChart.Styles.Shape(_chart.TChart.Chart);
            Chart = _chart;

            Name = name;

            //TeeChart Initialization
            TSeries.CustomHorizAxis = HorizontalAxis.TAxis;
            TSeries.CustomVertAxis = VerticalAxis.TAxis;

            Style = ShapeStyle.Rectangle;
            Visible = false;
            desiredVisible = true;

            //TSeries.FillSampleValues();
            TSeries.HorizAlignment = Steema.TeeChart.Styles.ShapeTextHorizAlign.Center;
            TSeries.VertAlignment = Steema.TeeChart.Styles.ShapeTextVertAlign.Center;
            TSeries.XYStyle = Steema.TeeChart.Styles.ShapeXYStyles.Axis;
            TSeries.XValues.Order = Steema.TeeChart.Styles.ValueListOrder.Ascending;
            TSeries.XValues.DataMember = "X";
            TSeries.YValues.DataMember = "Y";
            TSeries.Add(0, 0);
            TSeries.Add(0, 0);
            TSeries.Visible = false;

            //Events
            TSeries.Click += delegate(object sender, System.Windows.Forms.MouseEventArgs e)
            {
                if (Click != null)
                    Click(this, e);
            };
        }

        public static ShapeSeries Create(Chart _chart, string name, string unit)
        {
            ShapeSeries s = new ShapeSeries(_chart, name, unit);
            _chart.AddSeries(s);
            return s;
        }

        public override ISeries CreateMirror(Chart _chart)
        {
            ShapeSeries s = new ShapeSeries(_chart, Name, Unit);
            s.Color = this.Color;

            this.PointAdded += delegate(object sender, EventArgs e)
            {
                s.RefreshDataSource();
            };
            this.PointRemoved += delegate(object sender, EventArgs e)
            {
                s.RefreshDataSource();
            };

            s.tSeries.DataSource = tSeries;

            _chart.AddSeries(s);
            return s;
        }

        #endregion Construction

        public enum ShapeStyle  //Remember to add to ShapeStyle2TeeChart and TeeChart2ShapeStyle when adding to this enum!
        {
            Rectangle,
            Circle
        }
        private Steema.TeeChart.Styles.ShapeStyles ShapeStyle2TeeChart(ShapeStyle style)
        {
            switch (style)
            {
                case ShapeStyle.Circle: return Steema.TeeChart.Styles.ShapeStyles.Circle;
                default: return Steema.TeeChart.Styles.ShapeStyles.Rectangle;   //case ShapeStyle.Rectangle
            }
        }
        private ShapeStyle TeeChart2ShapeStyle(Steema.TeeChart.Styles.ShapeStyles style)
        {
            switch (style)
            {
                case Steema.TeeChart.Styles.ShapeStyles.Circle: return ShapeStyle.Circle;
                default: return ShapeStyle.Rectangle; //case Steema.TeeChart.Styles.ShapeStyles.Rectangle
            }
        }
    }
}
