﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Collections;

namespace Ellab.Charting
{
    public partial class Viewport : UserControl
    {
        #region Zooming

        public EventHandler ZoomInHandler
        {
            get { return chart.Zoom.ZoomInHandler; }
            set { chart.Zoom.ZoomInHandler = value; }
        }
        public EventHandler DefaultZoomInHandler
        {
            get { return chart.Zoom.DefaultZoomInHandler; }
        }
        //public EventHandler CustomZoomInHandler
        //{
        //    get { return chart.Zoom.CustomZoomInHandler; }
        //    set { chart.Zoom.CustomZoomInHandler = value; }
        //}
        //public EventHandler CustomDefaultZoomInHandler
        //{
        //    get { return chart.Zoom.CustomDefaultZoomInHandler; }
        //}
        public EventHandler ZoomOutHandler
        {
            get { return chart.Zoom.ZoomOutHandler; }
            set { chart.Zoom.ZoomOutHandler = value; }
        }
        public EventHandler DefaultZoomOutHandler
        {
            get { return chart.Zoom.DefaultZoomOutHandler; }
        }

        #endregion

        #region Aesthetics

        public string Title
        {
            get { return chart.Title; }
            set { chart.Title = value; }
        }

        public bool LegendVisible
        {
            get { return chart.LegendVisible; }
            set { chart.LegendVisible = value; }
        }

        #endregion

        #region Updating / Redrawing

        public bool SuspendUpdates
        {
            get { return chart.SuspendUpdates; }
            set { chart.SuspendUpdates = value; }
        }
        public void ForceUpdate()
        {
            chart.ForceUpdate();
        }

        #endregion

        #region Axes

        public Axis HorizontalAxis
        {
            get { return chart.HorizontalAxis; }
        }
        
        public AxisCollection VerticalAxes
        {
            get { return chart.VerticalAxes; }
        }

        #endregion Axes

        public Viewport(Chart original)
        {
            InitializeComponent();

            original.SeriesAdded += new EventHandler<SeriesEventArgs>(original_SeriesAdded);
            original.SeriesRemoved += new EventHandler<SeriesEventArgs>(original_SeriesRemoved);

            original.ToolAdded += new EventHandler<ToolEventArgs>(original_ToolAdded);
            original.ToolRemoved += new EventHandler<ToolEventArgs>(original_ToolRemoved);

            //Copy stuff from original
            chart.LegendVisible = original.LegendVisible;

            foreach (ISeries s in original.Series)
            {
                s.CreateMirror(chart);
            }
            foreach (ITool v in original.Tools)
            {
                v.CreateMirror(chart);
            }
        }

        #region Maintain copy of original chart

        private void original_SeriesAdded(object sender, SeriesEventArgs e)
        {
            e.Series.CreateMirror(chart);
        }
        private void original_SeriesRemoved(object sender, SeriesEventArgs e)
        {
            chart.RemoveSeries(e.Series.Name);
        }

        private void original_ToolAdded(object sender, ToolEventArgs e)
        {
            e.Tool.CreateMirror(chart);
        }
        private void original_ToolRemoved(object sender, ToolEventArgs e)
        {
            chart.RemoveTool(e.Tool.Name);
        }

        #endregion
    }
}